function [pi, q, v, vs, Bellman_error] = value_iteration(mdp, niters, initV)

%% input: mdp
%% output:
%%   pi: a optimal policy
%%   q: q-function
%%   v: value function
%%   vs: value function throughout iterations
%%   Bellman_error: maximum Bellman error, and mean Bellman error throughout iterations
%%   lambda: state action visitation rate for the optimal policy
%%
%%


num_actions = length(mdp.P);
num_states = length(mdp.P{1});


v = initV; v_new = zeros(num_states,1);
q = zeros(num_states,num_actions);

Bellman_error = zeros(niters,2);

vs = zeros(niters, num_states);

for vi_iter = 1:niters
	%	vi_iter
	for s=1:num_states
		for a=1:num_actions
			q(s,a) = mdp.r(s,a) + mdp.gamma * mdp.P{a}(s,:)*v;
		end
	end
	for s=1:num_states
		[v_new(s),pi(s)] = max(q(s,:));
	end
	Bellman_error(vi_iter,1) = max(abs(v_new - v));
	Bellman_error(vi_iter,2) = mean(abs(v_new-v));
	v = v_new;
	vs(vi_iter,:) = v';
end


